Struct EventDesc
Source pub struct EventDesc<T, S>{
pub name: T,
pub src: Vec<S>,
pub dst: S,
}
EventDesc represents an event when initializing the FSM.
name is the event name used when calling for a transition.
src is a slice of source states that the FSM must be in to perform a
state transition.
dst is the destination state that the FSM will be in if the transition
succeeds.
Formats the value using the given formatter.
Read more
Immutably borrows from an owned value.
Read more
Mutably borrows from an owned value.
Read more
Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From<T> for U chooses to do.
The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.