FSM

Struct FSM 

pub struct FSM<'a, S, I, F: Action<S, I>> { /* private fields */ }
FSM represents a finite state machine.

The FSM is initialized with an initial state and a list of events.

Implementations

Source

impl<'a, S, I, F> FSM<'a, S, I, F>
where S: FSMState, I: IntoIterator, F: Action<S, I>,

pub fn new<T>( initial: S, events: impl IntoIterator<Item = EventDesc<T, S>>, hooks: impl IntoIterator<Item = (HookType<T, S>, F)>, ) -> Self
where T: AsRef<str>,

new creates a new FSM.

pub fn get_current(&self) -> S

get_current returns the current state of the FSM.

pub fn on_event<T: AsRef<str>>( &mut self, event: T, args: Option<&I>, ) -> Result<(), FSMError<String>>

on_event initiates a state transition with the named event.

pub fn is<T: AsRef<S>>(&self, state: T) -> bool

is returns true if state is the current state.

pub fn can<T: AsRef<str>>(&self, event: T) -> bool

can returns true if event can occur in the current state.